import os
from pyhamilton import (HamiltonInterface, LayoutManager, ResourceType, Tip96,
    INITIALIZE, PICKUP96, EJECT96)

layfile = os.path.abspath(os.path.join('.', '96_head_pickup_eject.lay'))
lmgr = LayoutManager(layfile)

tip_name_from_line = lambda line: LayoutManager.layline_first_field(line)
tip_name_condition = lambda line: LayoutManager.field_starts_with(tip_name_from_line(line), 'STF_L_')
tips_type = ResourceType(Tip96, tip_name_condition, tip_name_from_line)
tips = lmgr.assign_unused_resource(tips_type)

if __name__ == '__main__':
    with HamiltonInterface() as hammy:
        print('INITIALIZED!!', hammy.wait_on_response(hammy.send_command(INITIALIZE)))
        labware = str(tips.layout_name())
        labware_poss = '; '.join([labware + ',' + str(i+1) for i in range(96)]) + ';'
         # A dictionary can be unpacked into the command...
        cmd_dict = {'labwarePositions':labware_poss}
        id = hammy.send_command(PICKUP96, **cmd_dict)
        print(hammy.wait_on_response(id, raise_first_exception=True))
         # Or the command fields can be specified with keyword arguments
        id = hammy.send_command(EJECT96, labwarePositions=labware_poss)
        print(hammy.wait_on_response(id, raise_first_exception=True))



import os
from pyhamilton import (HamiltonInterface, LayoutManager, ResourceType, Tip96,
    INITIALIZE, PICKUP, NoTipError, HardwareError)

layfile = os.path.abspath(os.path.join('.', 'minimal_error_example.lay'))
lmgr = LayoutManager(layfile)

tip_name_from_line = lambda line: LayoutManager.layline_first_field(line)
tip_name_condition = lambda line: LayoutManager.field_starts_with(tip_name_from_line(line), 'HTF_L_')
tips_type = ResourceType(Tip96, tip_name_condition, tip_name_from_line)
tips = lmgr.assign_unused_resource(tips_type)

if __name__ == '__main__':
    with HamiltonInterface() as hammy:
        print('INITIALIZED!!', hammy.wait_on_response(hammy.send_command(INITIALIZE)))
        try:
            id = hammy.send_command(PICKUP, labwarePositions=str(tips.layout_name()) + ', 1;')
            print(hammy.wait_on_response(id, raise_first_exception=True))
        except NoTipError:
            print('\n'*10 + 'THERE WAS NO TIP THERE' + '\n'*10)
        except HardwareError:
            print('\n'*10 + 'Did I just crash into something?' + '\n'*10)



import os
from pyhamilton import (HamiltonInterface, LayoutManager, ResourceType, Plate96,
    INITIALIZE, ISWAP_GET, ISWAP_PLACE,
    HamiltonError)

layfile = os.path.abspath(os.path.join('.', 'grip_move_plate.lay'))
lmgr = LayoutManager(layfile)

plate_type = ResourceType(Plate96, 'Cos_96_Rd_0001')
plate = lmgr.assign_unused_resource(plate_type)

target_site_type = ResourceType(Plate96, 'Cos_96_Rd_0002')
target_site = lmgr.assign_unused_resource(target_site_type)

if __name__ == '__main__':
    plate_pos = plate.layout_name() + ', ' + plate.position_id(0)
    target_pos = target_site.layout_name() + ', ' + target_site.position_id(0)
    with HamiltonInterface() as hammy:
        hammy.wait_on_response(hammy.send_command(INITIALIZE))
        for id in (hammy.send_command(ISWAP_GET, plateLabwarePositions=plate_pos),
                   hammy.send_command(ISWAP_PLACE, plateLabwarePositions=target_pos)):
            print(hammy.wait_on_response(id, raise_first_exception=True))
        for id in (hammy.send_command(ISWAP_GET, plateLabwarePositions=target_pos),
                   hammy.send_command(ISWAP_PLACE, plateLabwarePositions=plate_pos)):
            print(hammy.wait_on_response(id, raise_first_exception=True))



#!python3

import os
from pyhamilton import (HamiltonInterface, LayoutManager, ResourceType, Tip96, Plate96,
    INITIALIZE, PICKUP, EJECT, ASPIRATE, DISPENSE,
    HamiltonError)

layfile = os.path.abspath(os.path.join('.', 'single_ch_aspirate_dispense.lay'))
lmgr = LayoutManager(layfile)

tip_name_from_line = lambda line: LayoutManager.layline_first_field(line)
tip_name_condition = lambda line: LayoutManager.field_starts_with(tip_name_from_line(line), 'HTF_L_')
tips_type = ResourceType(Tip96, tip_name_condition, tip_name_from_line)
tips = lmgr.assign_unused_resource(tips_type)

plate_type = ResourceType(Plate96, 'Cos_96_Rd_0001')
plate = lmgr.assign_unused_resource(plate_type)

if __name__ == '__main__':
    tip_no = 88 # top right corner
    well_no = 7 # bottom left corner
    tip_labware_pos = tips.layout_name() + ', ' + tips.position_id(tip_no) + ';'
    well_labware_pos = plate.layout_name() + ', ' + plate.position_id(well_no) + ';'
    liq_class = 'HighVolumeFilter_Water_DispenseJet_Empty'
    with HamiltonInterface() as hammy:
        hammy.wait_on_response(hammy.send_command(INITIALIZE))
        ids = [hammy.send_command(PICKUP, labwarePositions=tip_labware_pos),
               hammy.send_command(ASPIRATE, labwarePositions=well_labware_pos, volumes=100.0, liquidClass=liq_class),
               hammy.send_command(DISPENSE, labwarePositions=well_labware_pos, volumes=100.0, liquidClass=liq_class),
               hammy.send_command(EJECT, labwarePositions=tip_labware_pos)]
        for id in ids:
            try:
                print(hammy.wait_on_response(id, raise_first_exception=True))
            except HamiltonError as he:
                print(he)



#!python3

from pyhamilton import HamiltonInterface, LayoutManager, INITIALIZE, WASH96_EMPTY

def wash_empty_refill(ham, **more_options):
    print('wash_empty_refill: empty the washer' +
            ('' if not more_options else ' with extra options ' + str(more_options)))
    ham.wait_on_response(ham.send_command(WASH96_EMPTY, **more_options))

if __name__ == '__main__':
    LayoutManager('with_washer.lay', install=True)
    with HamiltonInterface() as ham_int:
        print('Please wait, system starting--preparing to wash washer.\n')
        ham_int.wait_on_response(ham_int.send_command(INITIALIZE), raise_first_exception=True)
        wash_empty_refill(ham_int, refillAfterEmpty=1, chamber1WashLiquid=1, chamber2WashLiquid=1) # 1=both chambers; 1=Liquid 2 (water)
        wash_empty_refill(ham_int) # empty